@@ -16,25 +16,26 @@
 #endif
 
 #include "odr-priv.h"
+#include <yaz/log.h>
+#include <assert.h>
 
 int ber_octetstring(ODR o, Odr_oct *p, int cons)
 {
     int res, len;
-    const unsigned char *base;
-    unsigned char *c;
+    const char *base;
 
     switch (o->direction)
     {
     case ODR_DECODE:
-        if ((res = ber_declen(o->bp, &len, odr_max(o))) < 0)
+        if ((res = ber_declen(o->op->bp, &len, odr_max(o))) < 0)
         {
             odr_seterror(o, OPROTO, 14);
             return 0;
         }
-        o->bp += res;
+        o->op->bp += res;
         if (cons)       /* fetch component strings */
         {
-            base = o->bp;
+            base = o->op->bp;
             while (odp_more_chunks(o, base, len))
                 if (!odr_octetstring(o, &p, 0, 0))
                     return 0;
@@ -51,21 +52,11 @@ int ber_octetstring(ODR o, Odr_oct *p, int cons)
             odr_seterror(o, OOTHER, 16);
             return 0;
         }
-        if (len + 1 > p->size - p->len)
-        {
-            c = (unsigned char *)odr_malloc(o, p->size += len + 1);
-            if (p->len)
-                memcpy(c, p->buf, p->len);
-            p->buf = c;
-        }
-        if (len)
-            memcpy(p->buf + p->len, o->bp, len);
-        p->len += len;
-        o->bp += len;
-        /* the final null is really not part of the buffer, but */
-        /* it helps somes applications that assumes C strings */
-        if (len)
-            p->buf[p->len] = '\0';
+        p->len = len;
+        p->buf = odr_malloc(o, len + 1);
+        memcpy(p->buf, o->op->bp, len);
+        p->buf[len] = '\0';
+        o->op->bp += len;
         return 1;
     case ODR_ENCODE:
         if ((res = ber_enclen(o, p->len, 5, 0)) < 0)
