@@ -62,12 +62,11 @@ size_t odr_total(ODR o)
     return nmem_total(o->mem);
 }
 
-Odr_oct *odr_create_Odr_oct(ODR o, const unsigned char *buf, int sz)
+Odr_oct *odr_create_Odr_oct(ODR o, const char *buf, int sz)
 {
     Odr_oct *p = (Odr_oct *) odr_malloc(o, sizeof(Odr_oct));
-    p->buf = (unsigned char *) odr_malloc(o, sz);
+    p->buf = (char *) odr_malloc(o, sz);
     memcpy(p->buf, buf, sz);
-    p->size = sz;
     p->len = sz;
     return p;
 }
@@ -81,52 +80,47 @@ int odr_grow_block(ODR b, int min_bytes)
 
     if (!b->op->can_grow)
         return -1;
-    if (!b->size)
+    if (!b->op->size)
         togrow = 1024;
     else
-        togrow = b->size;
+        togrow = b->op->size;
     if (togrow < min_bytes)
         togrow = min_bytes;
-    if (b->size && !(b->buf =
-                     (unsigned char *) xrealloc(b->buf, b->size += togrow)))
+    if (b->op->size && !(b->op->buf =
+                     (char *) xrealloc(b->op->buf, b->op->size += togrow)))
         abort();
-    else if (!b->size && !(b->buf = (unsigned char *)
-                           xmalloc(b->size = togrow)))
+    else if (!b->op->size && !(b->op->buf = (char *)
+                               xmalloc(b->op->size = togrow)))
         abort();
     return 0;
 }
 
-int odr_write2(ODR o, const char *buf, int bytes)
+int odr_write(ODR o, const char *buf, int bytes)
 {
-    if (o->pos + bytes >= o->size && odr_grow_block(o, bytes))
+    if (o->op->pos + bytes >= o->op->size && odr_grow_block(o, bytes))
     {
         odr_seterror(o, OSPACE, 40);
         return -1;
     }
-    memcpy(o->buf + o->pos, buf, bytes);
-    o->pos += bytes;
-    if (o->pos > o->top)
-        o->top = o->pos;
+    memcpy(o->op->buf + o->op->pos, buf, bytes);
+    o->op->pos += bytes;
+    if (o->op->pos > o->op->top)
+        o->op->top = o->op->pos;
     return 0;
 }
 
-int odr_write(ODR o, unsigned char *buf, int bytes)
-{
-    return odr_write2(o, (char *) buf, bytes);
-}
-
 int odr_seek(ODR o, int whence, int offset)
 {
     if (whence == ODR_S_CUR)
-        offset += o->pos;
+        offset += o->op->pos;
     else if (whence == ODR_S_END)
-        offset += o->top;
-    if (offset > o->size && odr_grow_block(o, offset - o->size))
+        offset += o->op->top;
+    if (offset > o->op->size && odr_grow_block(o, offset - o->op->size))
     {
         odr_seterror(o, OSPACE, 41);
         return -1;
     }
-    o->pos = offset;
+    o->op->pos = offset;
     return 0;
 }
 
